@@ -52,7 +52,7 @@ my $COMMENT = qr/\# [^\n;]* (?=[;\n])?/xms;
 
 my $CODE    = qr/ (?: (?: $STRING | [^'"] )*? ) /xms; # ' for poor editors
 
-has symbol_table => (
+has symbol_table => ( # the global symbol table
     is  => 'ro',
     isa => 'HashRef',
 
@@ -65,6 +65,9 @@ has scope => (
     is  => 'rw',
     isa => 'ArrayRef[HashRef]',
 
+    clearer => 'init_scope',
+
+    lazy    => 1,
     default => sub{ [ {} ] },
 
     init_arg => undef,
@@ -190,6 +193,7 @@ sub split {
             last;
         }
     }
+
     ## tokens: @tokens
     return \@tokens;
 }
@@ -308,11 +312,14 @@ sub lex {
 sub parse {
     my($parser, $input, %args) = @_;
 
-    $parser->input( $parser->preprocess($input) );
-
     $parser->file( $args{file} // '<input>' );
     $parser->line( $args{line} // 0 );
     $parser->near_token('(start)');
+    $parser->init_scope();
+
+    local $parser->{symbol_table} = { %{ $parser->symbol_table } };
+
+    $parser->input( $parser->preprocess($input) );
 
     my $ast = $parser->statements();
     if($parser->input ne '') {
@@ -320,6 +327,7 @@ sub parse {
         $parser->_error("Syntax error");
     }
     $parser->near_token(undef);
+
     return $ast;
 }
 
@@ -338,17 +346,24 @@ sub _define_basic_symbols {
     $parser->symbol('(end)')->is_end(1); # EOF
 
     $parser->symbol('(name)');
+
     my $s = $parser->symbol('(variable)');
     $s->arity('variable');
-    $s->set_nud(\&_nud_literal);
+    $s->set_nud(\&nud_literal);
 
-    $parser->symbol('(literal)')->set_nud(\&_nud_literal);
+    $parser->symbol('(literal)')->set_nud(\&nud_literal);
 
     $parser->symbol(';');
+    $parser->symbol('(');
+    $parser->symbol(')');
+    $parser->symbol(',');
 
-    # basic commands
-    $parser->symbol('print')    ->set_std(\&_std_command);
-    $parser->symbol('print_raw')->set_std(\&_std_command);
+    # common commands
+    $parser->symbol('print')    ->set_std(\&std_command);
+    $parser->symbol('print_raw')->set_std(\&std_command);
+
+    # common constants
+    $parser->define_constant('nil', undef);
 
     return;
 }
@@ -356,6 +371,12 @@ sub _define_basic_symbols {
 sub define_basic_operators {
     my($parser) = @_;
 
+    $parser->infix('(', 256, \&led_call);
+    $parser->infix('.', 256, \&led_dot);
+    $parser->infix('[', 256, \&led_fetch);
+
+    $parser->prefix('(', 200, \&nud_paren);
+
     $parser->prefix('!', 200);
     $parser->prefix('+', 200);
     $parser->prefix('-', 200);
@@ -386,7 +407,7 @@ sub define_basic_operators {
     $parser->infix('max', 120);
 
     $parser->symbol(':');
-    $parser->infix('?', 110, \&_led_ternary);
+    $parser->infix('?', 110, \&led_ternary);
 
     $parser->assignment('=',   100);
     $parser->assignment('+=',  100);
@@ -409,9 +430,7 @@ sub define_basic_operators {
 sub define_symbols {
     my($parser) = @_;
 
-    # separators
-    $parser->symbol(',');
-    $parser->symbol(')');
+    # syntax specific separators
     $parser->symbol(']');
     $parser->symbol('}')->is_end(1); # block end
     $parser->symbol('->');
@@ -422,32 +441,23 @@ sub define_symbols {
     # operators
     $parser->define_basic_operators();
 
-    $parser->infix('.', 256, \&_led_dot);
-    $parser->infix('[', 256, \&_led_fetch);
-    $parser->infix('(', 256, \&_led_call);
-
-    $parser->prefix('(', 200, \&_nud_paren);
-
-    # constants
-    $parser->define_constant('nil', undef);
-
     # statements
-    $parser->symbol('{')        ->set_std(\&_std_block);
-    $parser->symbol('if')       ->set_std(\&_std_if);
-    $parser->symbol('for')      ->set_std(\&_std_for);
-    $parser->symbol('while' )   ->set_std(\&_std_while);
+    $parser->symbol('{')        ->set_std(\&std_block);
+    $parser->symbol('if')       ->set_std(\&std_if);
+    $parser->symbol('for')      ->set_std(\&std_for);
+    $parser->symbol('while' )   ->set_std(\&std_while);
 
-    $parser->symbol('include')  ->set_std(\&_std_command);
+    $parser->symbol('include')  ->set_std(\&std_command);
 
     # template inheritance
 
-    $parser->symbol('cascade')  ->set_std(\&_std_bare_command);
-    $parser->symbol('macro')    ->set_std(\&_std_proc);
-    $parser->symbol('block')    ->set_std(\&_std_proc);
-    $parser->symbol('around')   ->set_std(\&_std_proc);
-    $parser->symbol('before')   ->set_std(\&_std_proc);
-    $parser->symbol('after')    ->set_std(\&_std_proc);
-    $parser->symbol('super')    ->set_std(\&_std_marker);
+    $parser->symbol('cascade')  ->set_std(\&std_cascade);
+    $parser->symbol('macro')    ->set_std(\&std_proc);
+    $parser->symbol('block')    ->set_std(\&std_proc);
+    $parser->symbol('around')   ->set_std(\&std_proc);
+    $parser->symbol('before')   ->set_std(\&std_proc);
+    $parser->symbol('after')    ->set_std(\&std_proc);
+    $parser->symbol('super')    ->set_std(\&std_marker);
 
     return;
 }
@@ -557,7 +567,7 @@ sub expression_list {
     return \@args;
 }
 
-sub _led_infix {
+sub led_infix {
     my($parser, $symbol, $left) = @_;
     my $bin = $symbol->clone(arity => 'binary');
 
@@ -569,11 +579,11 @@ sub _led_infix {
 sub infix {
     my($parser, $id, $bp, $led) = @_;
 
-    $parser->symbol($id, $bp)->set_led($led || \&_led_infix);
+    $parser->symbol($id, $bp)->set_led($led || \&led_infix);
     return;
 }
 
-sub _led_infixr {
+sub led_infixr {
     my($parser, $symbol, $left) = @_;
     my $bin = $symbol->clone(arity => 'binary');
     $bin->first($left);
@@ -584,11 +594,11 @@ sub _led_infixr {
 sub infixr {
     my($parser, $id, $bp, $led) = @_;
 
-    $parser->symbol($id, $bp)->set_led($led || \&_led_infixr);
+    $parser->symbol($id, $bp)->set_led($led || \&led_infixr);
     return;
 }
 
-sub _led_assignment {
+sub led_assignment {
     my($parser, $symbol, $left) = @_;
 
     $parser->near_token($left);
@@ -598,11 +608,11 @@ sub _led_assignment {
 sub assignment {
     my($parser, $id, $bp) = @_;
 
-    $parser->symbol($id, $bp)->set_led(\&_led_assignment);
+    $parser->symbol($id, $bp)->set_led(\&led_assignment);
     return;
 }
 
-sub _led_ternary {
+sub led_ternary {
     my($parser, $symbol, $left) = @_;
 
     my $cond = $symbol->clone(arity => 'ternary');
@@ -614,15 +624,25 @@ sub _led_ternary {
     return $cond;
 }
 
-sub _led_dot {
+sub is_valid_field {
+    my($parser, $token) = @_;
+    given($token->arity) {
+        when("name") {
+            return 1;
+        }
+        when("literal") {
+            return Mouse::Util::TypeConstraints::Int($token->id);
+        }
+    }
+    return 0;
+}
+
+sub led_dot {
     my($parser, $symbol, $left) = @_;
 
     my $t = $parser->token;
-    if($t->arity ne "name") {
-        if(!($t->arity eq "literal"
-                && Mouse::Util::TypeConstraints::Int($t->id))) {
-            $parser->_error("Expected a field name but $t");
-        }
+    if(!$parser->is_valid_field($t)) {
+        $parser->_error("Expected a field name but $_ ($t)");
     }
 
     my $dot = $symbol->clone(arity => 'binary');
@@ -630,9 +650,9 @@ sub _led_dot {
     $dot->first($left);
     $dot->second($t->clone(arity => 'literal'));
 
-    $parser->advance();
+    $t = $parser->advance();
 
-    if($parser->token->id eq "(") {
+    if($t->id eq "(") {
         $parser->advance(); # "("
         $dot->third( $parser->expression_list() );
         $parser->advance(")");
@@ -642,7 +662,7 @@ sub _led_dot {
     return $dot;
 }
 
-sub _led_fetch {
+sub led_fetch {
     my($parser, $symbol, $left) = @_;
 
     my $fetch = $symbol->clone(arity => 'binary');
@@ -654,7 +674,7 @@ sub _led_fetch {
     return $fetch;
 }
 
-sub _led_call {
+sub led_call {
     my($parser, $symbol, $left) = @_;
 
     my $call = $symbol->clone(arity => 'call');
@@ -666,7 +686,7 @@ sub _led_call {
     return $call;
 }
 
-sub _nud_prefix {
+sub nud_prefix {
     my($parser, $symbol) = @_;
     my $un = $symbol->clone(arity => 'unary');
     $parser->reserve($un);
@@ -679,11 +699,11 @@ sub prefix {
 
     my $symbol = $parser->symbol($id);
     $symbol->ubp($bp);
-    $symbol->set_nud($nud || \&_nud_prefix);
+    $symbol->set_nud($nud || \&nud_prefix);
     return;
 }
 
-sub _nud_constant {
+sub nud_constant {
     my($parser, $symbol) = @_;
 
     my $c = $symbol->clone(arity => 'literal');
@@ -696,7 +716,7 @@ sub define_constant {
     my($parser, $id, $value) = @_;
 
     my $symbol = $parser->symbol($id);
-    $symbol->set_nud(\&_nud_constant);
+    $symbol->set_nud(\&nud_constant);
     $symbol->value($value);
     return;
 }
@@ -731,14 +751,14 @@ sub find { # find a name from all the scopes
 sub reserve { # reserve a name to the scope
     my($parser, $symbol) = @_;
     if($symbol->arity ne 'name' or $symbol->reserved) {
-        return;
+        return $symbol;
     }
 
     my $top = $parser->scope->[-1];
     my $t = $top->{$symbol->id};
     if($t) {
         if($t->reserved) {
-            return;
+            return $symbol;
         }
         if($t->arity eq "name") {
            confess("Already defined: $symbol");
@@ -746,7 +766,7 @@ sub reserve { # reserve a name to the scope
     }
     $top->{$symbol->id} = $symbol;
     $symbol->reserved(1);
-    return;
+    return $symbol;
 }
 
 sub define { # define a name to the scope
@@ -761,7 +781,7 @@ sub define { # define a name to the scope
     $top->{$symbol->id} = $symbol;
 
     $symbol->reserved(0);
-    $symbol->set_nud(\&_nud_literal);
+    $symbol->set_nud(\&nud_literal);
     $symbol->remove_led();
     $symbol->remove_std();
     $symbol->lbp(0);
@@ -770,11 +790,10 @@ sub define { # define a name to the scope
 }
 
 
-sub _nud_function{
+sub nud_function{
     my($p, $s) = @_;
     my $f = $s->clone(arity => 'function');
-    $p->reserve($f);
-    return $f;
+    return $p->reserve($f);
 }
 
 sub define_function {
@@ -782,17 +801,16 @@ sub define_function {
 
     foreach my $name(@names) {
         my $symbol = $compiler->symbol($name);
-        $symbol->set_nud(\&_nud_function);
+        $symbol->set_nud(\&nud_function);
         $symbol->value($name);
     }
     return;
 }
 
-sub _nud_macro{
+sub nud_macro{
     my($p, $s) = @_;
     my $f = $s->clone(arity => 'macro');
-    $p->reserve($f);
-    return $f;
+    return $p->reserve($f);
 }
 
 sub define_macro {
@@ -800,7 +818,7 @@ sub define_macro {
 
     foreach my $name(@names) {
         my $symbol = $compiler->symbol($name);
-        $symbol->set_nud(\&_nud_macro);
+        $symbol->set_nud(\&nud_macro);
         $symbol->value($name);
     }
     return;
@@ -823,8 +841,8 @@ sub statement { # process one or more statements
     }
 
     if($t->has_std) { # is $t a statement?
-        $parser->advance();
         $parser->reserve($t);
+        $parser->advance();
         return $t->std($parser);
     }
 
@@ -868,19 +886,19 @@ sub block {
 }
 
 
-sub _nud_literal {
+sub nud_literal {
     my($parser, $symbol) = @_;
     return $symbol; # as is
 }
 
-sub _nud_paren {
+sub nud_paren {
     my($parser, $symbol) = @_;
     my $expr = $parser->expression(0);
     $parser->advance(')');
     return $expr;
 }
 
-sub _std_block {
+sub std_block {
     my($parser, $symbol) = @_;
     $parser->new_scope();
     my $a = $parser->statements();
@@ -889,7 +907,7 @@ sub _std_block {
     return $a;
 }
 
-#sub _std_var {
+#sub std_var {
 #    my($parser, $symbol) = @_;
 #    my @a;
 #    while(1) {
@@ -921,7 +939,7 @@ sub _std_block {
 
 # -> VARS { STATEMENTS }
 # ->      { STATEMENTS }
-sub _pointy {
+sub pointy {
     my($parser, $node) = @_;
 
     $parser->advance("->");
@@ -933,13 +951,17 @@ sub _pointy {
 
         $parser->advance("(") if $paren;
 
-        while((my $t = $parser->token)->arity eq "variable") {
+        my $t = $parser->token;
+        while($t->arity eq "variable") {
             push @vars, $t;
             $parser->define($t);
-            $parser->advance();
+            $t = $parser->advance();
 
-            if($parser->token->id eq ",") {
-                $parser->advance(); # ","
+            if($t->id eq ",") {
+                $t = $parser->advance(); # ","
+            }
+            else {
+                last;
             }
         }
 
@@ -955,25 +977,25 @@ sub _pointy {
     return;
 }
 
-sub _std_for {
+sub std_for {
     my($parser, $symbol) = @_;
 
     my $proc = $symbol->clone(arity => 'for');
     $proc->first( $parser->expression(0) );
-    $parser->_pointy($proc);
+    $parser->pointy($proc);
     return $proc;
 }
 
-sub _std_while {
+sub std_while {
     my($parser, $symbol) = @_;
 
     my $proc = $symbol->clone(arity => 'while');
     $proc->first( $parser->expression(0) );
-    $parser->_pointy($proc);
+    $parser->pointy($proc);
     return $proc;
 }
 
-sub _std_proc {
+sub std_proc {
     my($parser, $symbol) = @_;
 
     my $proc = $symbol->clone(arity => "proc");
@@ -985,11 +1007,11 @@ sub _std_proc {
     $parser->define_macro($name->id);
     $proc->first( $name->id );
     $parser->advance();
-    $parser->_pointy($proc);
+    $parser->pointy($proc);
     return $proc;
 }
 
-sub _std_if {
+sub std_if {
     my($parser, $symbol) = @_;
 
     my $if = $symbol->clone(arity => "if");
@@ -997,17 +1019,33 @@ sub _std_if {
     $if->first( $parser->expression(0) );
     $if->second( $parser->block() );
 
-    if($parser->token->id eq "else") {
-        $parser->advance(); # "else"
-        $parser->reserve($parser->token);
-        $if->third( $parser->token->id eq "if"
-            ? $parser->statement()
-            : $parser->block ());
+    my $top_if = $if;
+
+    my $t = $parser->token;
+    while($t->id eq "elsif") {
+        $parser->reserve($t);
+        $parser->advance(); # "elsif"
+
+        my $elsif = $t->clone(arity => "if");
+        $elsif->first(  $parser->expression(0) );
+        $elsif->second( $parser->block() );
+        $if->third([$elsif]);
+        $if = $elsif;
+        $t  = $parser->token;
     }
-    return $if;
+
+    if($t->id eq "else") {
+        $parser->reserve($t);
+        $t = $parser->advance(); # "else"
+
+        $if->third( $t->id eq "if"
+            ? $parser->statement()
+            : $parser->block());
+    }
+    return $top_if;
 }
 
-sub _std_command {
+sub std_command {
     my($parser, $symbol) = @_;
     my $args;
     if($parser->token->id ne ";") {
@@ -1056,10 +1094,13 @@ sub _get_bare_name {
     return \@parts;
 }
 
-sub _std_bare_command {
+sub std_cascade {
     my($parser, $symbol) = @_;
 
-    my $name = $parser->_get_bare_name();
+    my $base;
+    if($parser->token->id ne "with") {
+        $base = $parser->_get_bare_name();
+    }
     my @components;
 
     if($parser->token->id eq "with") {
@@ -1074,13 +1115,13 @@ sub _std_bare_command {
     }
     $parser->advance(";");
     return $symbol->clone(
-        first  => $name,
+        first  => $base,
         second => \@components,
-        arity  => 'bare_command');
+        arity  => 'cascade');
 }
 
 # markers for the compiler
-sub _std_marker {
+sub std_marker {
     my($parser, $symbol) = @_;
     $parser->advance(';');
     return $symbol->clone(arity => 'marker');
@@ -1089,7 +1130,7 @@ sub _std_marker {
 sub _error {
     my($self, $message) = @_;
 
-    Carp::croak(sprintf 'Xslate::Parser(%s:%d): %s%s',
+    Carp::croak(sprintf 'Xslate::Parser(%s:%d): %s%s while parsing templates',
         $self->file, $self->line+1, $message,
         $self->near_token ne ';' ? ", near '" . $self->near_token . "'" : '');
 }
